importFrom("graphics", "hist", "lines", "matplot", "plot", "points", "text")
importFrom("grid", "gpar", "grid.lines", "grid.polygon", "unit")
importFrom("grDevices", "axisTicks", "hcl", "rgb")
importFrom("lattice", "barchart", "do.breaks", "histogram", "lattice.getOption", "llines",
           "panel.abline", "panel.barchart", "panel.rect", "panel.segments", "panel.xyplot",
           "parallelplot", "trellis.par.get", "xyplot")
import("methods")
importFrom("modeltools", "clusters", "getModel", "ICL", "KLdiv", "Lapply",
           "parameters", "posterior", "prior", "refit", "relabel")
importFrom("nnet", "multinom", "nnet.default")
importFrom("stats", "AIC", "as.formula", "BIC", "binomial",
           ".checkMFClasses", "coef", "cov.wt", "dbinom", "delete.response", "dexp", 
           "dgamma", "dlnorm", "dnorm", "dpois", "dweibull", "factanal", "Gamma", ".getXlevels",
           "glm.fit", "lm.wfit", "model.frame", "model.matrix", "model.response",
           "na.omit", "optim", "pnorm", "poisson", "predict", "printCoefmat", "qnorm",
           "quantile", "rbinom", "residuals", "rgamma", "rmultinom", "rnorm",
           "rpois", "runif", "terms", "update", "update.formula", "var",
           "weighted.mean")
importFrom("stats4", "logLik", "nobs", "plot", "summary")
importFrom("utils", "getS3method")

export(
       "ExLinear",
       "ExNPreg",
       "ExNclus",
       "FLXdist",
       "FLXgradlogLikfun",
       "FLXlogLikfun",
       "FLXMCdist1",
       "FLXMCfactanal",
       "FLXMCmvbinary",
       "FLXMCmvcombi",
       "FLXMCmvnorm",
       "FLXMCmvpois",
       "FLXMCnorm1",
       "FLXPmultinom",
       "FLXPconstant",
       "FLXfit",
       "FLXglm",
       "FLXglmFix",
       "FLXbclust",
       "FLXmclust",
       "FLXconstant",
       "FLXmultinom",
       "FLXMRcondlogit",
       "FLXMRglm",
       "FLXMRglmfix",
       "FLXMRglmnet",
       "FLXMRlmer",
       "FLXMRlmm",
       "FLXMRlmmc",
       "FLXMRmgcv",
       "FLXMRmultinom",
       "FLXMRrobglm",
       "FLXMRziglm",
       "FLXgetDesign",
       "FLXgetParameters",
       "FLXreplaceParameters",
       "boot",
       "existGradient",
       "flexmix",
       "flxglht",
       "group",
       "initFlexmix",
       "LR_test",
       "plotEll",
       "refit_optim",
       "relabel",
       "stepFlexmix"
       )


exportClasses(
              "FLXcomponent",
              "FLXP",
              "FLXPmultinom",
              "FLXcontrol",
              "FLXdist",
              "FLXMRfix",
              "FLXMRglmfix",
              "FLXMRglm",
              "FLXMRlmer",
              "FLXMRlmm",
              "FLXMRlmmfix",
              "FLXMRlmmc",
              "FLXMRlmc",
              "FLXMRlmmcfix",
              "FLXMRlmcfix",
              "FLXMRmgcv",
              "FLXMRrobglm",
              "FLXMRziglm",
              "FLXM",
              "FLXMR",
              "FLXMC",
              "FLXMCsparse",
              "FLXnested",             
              "FLXR",
              "FLXRoptim",
              "FLXRmstep",
              "flexmix",
              "listOrdata.frame",
              "stepFlexmix",
              "summary.flexmix"
              )


exportMethods(
              "EIC",
              "FLXcheckComponent",
              "FLXdeterminePostunscaled",
              "FLXfit",
              "FLXgetK",
              "FLXgetModelmatrix",
              "FLXgetObs",
              "FLXmstep",       
              "FLXremoveComponent",
              "KLdiv",
              "ICL",
              "Lapply",
              "coerce",
              "clusters",
              "fitted",
              "flexmix",
              "flxglht",
              "getModel",
              "initialize",
              "logLik",
              "parameters",
              "plot",
              "posterior",
              "predict",
              "prior",
              "rFLXM",
              "refit",
              "relabel",
              "rflexmix",
              "show",
              "summary",
              "unique"
              )

S3method("as.data.frame", "simple_triplet_matrix")
