%
%  Copyright (C) 2004-2015 Friedrich Leisch and Bettina Gruen
%  $Id: BIC-methods.Rd 5008 2015-01-13 20:30:25Z gruen $
%
\name{BIC-methods}
\docType{methods}
\title{Methods for Function BIC}
\alias{BIC,flexmix-method}
\alias{BIC,stepFlexmix-method}
\description{Compute the Bayesian Information Criterion.}
\section{Methods}{
  \describe{
    \item{object = flexmix:}{Compute the BIC of a \code{flexmix} object}
    \item{object = stepFlexmix:}{Compute the BIC of all
      models contained in the \code{stepFlexmix} object.}
  }
}
\keyword{methods}
